;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
;
; File Notes:
; 1. The SPI2 module is configured to communicate with the PIC18F242
;    on the expansion board. The 18F242 will control the LCD on the
;    expansion board.
;
;END_HEADER


        .include "p30f6014.inc"

        .global _LCD_Controller_init    ;Provide global scope to routines
        .global _DisplayLCD
        .global LCDStringLine1, LCDStringLine2

	.section .bss
negtempflag:            .space 2

        .section        .data
        .align 2
LCDStringLine1: .ascii " Masters 04- 806P30 "
LCDStringLine2:	.ascii "lab2:a/d, dci & uart"
LCDStringLine3: .ascii "rp1="
lcd_RP1:        .ascii "0.00v rp2="
lcd_RP2:        .ascii "0.00v "
LCDStringLine4: .ascii "rp3="
lcd_RP3:        .ascii "0.00v tmp="
lcd_temp:	.ascii "+00dgC"

        .section        .text           ;Places source code in Code section

_LCD_Controller_init:

        push.d  w0

;Ensure LCD Controller is up and running
        mov     #256, w0
        mov     #7080, w1
        do      w1, end_delay1
        repeat  w0
                nop
                nop
end_delay1:     nop

;Reset LCD Controller
        bclr    SPI2STAT, #SPIROV               ;Clear overflow flag
        bset    LATG, #LATG9                    ;Deselect Slave LCD controller
        bclr    LATG, #LATG9                    ;Select Slave LCD controller

        mov.b   SPI2BUF, WREG
        mov.b   #0x80, w0
        mov.b   WREG, SPI2BUF
poll_port:
        btss    SPI2STAT, #SPIRBF
        bra     poll_port
        mov.b   SPI2BUF, WREG

;Ensure LCD Controller is up and running again
        mov     #256, w0
        mov     #7080, w1
        do      w1, end_delay2
        repeat  w0
                nop
                nop
end_delay2:     nop


;Enable SPI Interrupt Processing
        bclr    IFS1, #SPI2IF
        bset    IEC1, #SPI2IE                   ; Enable SPI2 ISR processing


        rcall   _DisplayLCD

        pop.d   w0

        return                                  ;Return to calling routine

_DisplayLCD:

        push.d  w0
	rcall	_UpdateDispBuffers

        bclr    SPI2STAT, #SPIROV       ;Clear overflow flag
        bset    LATG, #LATG9            ;Deselect Slave LCD controller
        bclr    LATG, #LATG9            ;Select Slave LCD controller

        clr     BypassCtrl
        setm    CtrlOrData

        mov     #80, w0
        mov     WREG, LcdCharCnt

        mov     #LCDStringLine1, w0
        mov     w0, LCDDataPtr

        mov.b   SPI2BUF, WREG
        mov.b   #0x82, w0
        mov.b   WREG, SPI2BUF

wait1:  cp0     LcdCharCnt
        bra     nz, wait1
        pop.d   w0

        return                          ;Return to calling routine

_UpdateDispBuffers:
        mov     PotRP1, w0
        rcall   _adcvolt
        mov     #lcd_RP1, w1               ;Update Pot RP1 decimal ASCII value on LCD
        rcall   refresh_adc_dispstr

        mov     PotRP2, w0
        rcall   _adcvolt
        mov     #lcd_RP2, w1               ;Update Pot RP2 decimal ASCII value on LCD
        rcall   refresh_adc_dispstr

        mov     PotRP3, w0
        rcall   _adcvolt
        mov     #lcd_RP3, w1               ;Update Pot RP3 decimal ASCII value on LCD
        rcall   refresh_adc_dispstr      ;Call routine present downstream in this file

        clr     negtempflag                ;Is the temperature Negative?
        mov     TempSensor, w0               ;If so ensure that the "-" character appears
        sub     #0x0199, w0
        bra     c, not_subzerotemp
        mov     TempSensor, w0
        mov     #0x0199, w1                ;0x199 is equal to 500 mv in 12-bit 5v ref. A/D
        subr    w0, w1, w0
        setm    negtempflag
not_subzerotemp:
        rcall   _adcvolt
        mov     #lcd_temp, w1
        mov     #0x002B,w0
        btsc    negtempflag, #0
        inc2    w0, w0
        mov.b   w0, [w1++]
        mov.b   _adtenths, WREG
        mov.b   w0, [w1++]
        mov.b   _adhundredths, WREG
        mov.b   w0, [w1]	
	return

refresh_adc_dispstr:
        mov.b   _adones, WREG
        mov.b   w0, [w1++]
        inc     w1, w1
        mov.b   _adtenths, WREG
        mov.b   w0, [w1++]
        mov.b   _adhundredths, WREG
        mov.b   w0, [w1]
        return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.end                                    ;EOF

