;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.60            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
; 1. This file contains the lab2_part2 subroutine that configures the dsPIC
;    12-bit Data Converter Interface (DCI) module
; 2. This file also contains the DCI interrupt service routine (ISR) named -
;    "__DCIInterrupt"
;
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab2.inc"             ;Lab-specific global equates

        ;Constants used to initialize the DCI module's Bit clock Generator
        .equ    FS, 7200                ;FS = Sampling Rate = 7200 Hz
        .equ    FSCKD, FS * 256         ;FSCKD = Bit Clock Rate = 256*FS
        .equ    BCG,  (FCY/(2*FSCKD))-1 ;Equation for DCI Baud Rate Generator
                                        ;from Family Reference Manual
        .equ    NUM_DCI_BUF, 4
        .equ    NUMSAMP, 256

                                        ;Provide global definitions
        .global lab2_part2              ;Provide global scope to "lab2_part2"
        .global __DCIInterrupt          ;Provide global scope to DCI ISR
                                        ;This is required so that the IVT
                                        ;is populated by the DCI ISR address.

        .section .data
SineTablePtr: .hword SineTable          ;Data variable in RAM that contains
                                        ;address of next tone sample to be
                                        ;transmitted by the DCI module
                                        ;-initialized to the first sample in
                                        ;the "SineTable" array

        .section .text                  ;Code Section in Program memory
lab2_part2:

        rcall _setup_Si3000_codec       ;Call subroutine in the library file,
                                        ;lab2.a, to set up the Si3000 Codec

;DCI Module Initialization
        clr     DCICON1                 ;Place DCI Module to a known state
        clr     DCICON2                 ;by clearing all control registers
        clr     DCICON3
        clr     TSCON
        clr     RSCON

        ;DCICON1 Initialization - Perform SFR initialization in 2 MOV
        ;instructions or a few "Bit-wise" operations (BSET/BCLR).
        bclr    DCICON1, #COFSM0        ;Set Frame sync mode (bits 1:0)
        bclr    DCICON1, #COFSM1        ;for multichannel frame sync mode
        bclr    DCICON1, #DJST          ;Set up Data justification control so
                                        ;data transmit/receive begins 1 bit-
                                        ;clock after the Fs pulse
        bclr    DCICON1, #CSCKE         ;Set up the bit-clock edge control bit
                                        ;so data changes on rising, and is
                                        ;sampled on falling
        bset    DCICON1, #CSDOM         ;Tristate output on CSDO pin when DCI
                                        ;module is not transmitting
        bclr    DCICON1, #COFSD         ;Set up Frame sync to be generated by the DCI
        bclr    DCICON1, #CSCKD         ;Set up Bit-Clock to be generated by DCI
                                        ;DCI Module not enabled yet

        ;DCICON2 Initialization - Perform SFR initialization in 2 MOV
        ;instructions or a few "Bit-wise" operations (BSET/BCLR).
        mov     #0b0000000111101111, w0 ;Configure data word size to 16-bits
        mov     w0, DCICON2             ;Configure for a data frame of 16 words

        bset    DCICON2, #BLEN1         ;Set buffer length (4 data words
        bset    DCICON2, #BLEN0         ;are buffered between interrupts)

        ;DCICON3 Initialization
        mov     #BCG, W0                ;Initialize the DCI bit clock generator
        mov     W0, DCICON3             ;using the BCG equate at the top of
                                        ;this file

        ;RSCON Initialization
        bset    RSCON, #0               ;Enable reception on time slot 0

        ;TSCON Initialization
        bset    TSCON, #0               ;Enable transmission on time slot 0

        ;DCI Interrupt Set up
        bclr    IFS2, #DCIIF            ;Clear the DCI interrupt flag
        bset    IEC2, #DCIIE            ;Enable DCI interrupts

        ;Turn on the DCI Module
        bset    DCICON1, #DCIEN         ;Enable the DCI module now

        return                          ;Return to calling routine

;...
;DCI Interrupt Service Routine
;Name: __DCIInterrupt
;Scope: Global
;FAQs:
;     1. Want to use a different name for your DCI ISR?
;        - Modify the name in the linker script
;     2. Why the double underscore?
;        - For compatibility with the C30 compiler & to avoid namespace issues
;     3. Why global scope for the ISR?
;        - So the linker can place the address into the IVT (Vector table)
;...

__DCIInterrupt:
        push.d  w0                      ;Save w0, w1 on to the stack
        push.d  w2                      ;Save w2, w3 on to the stack
        push    w4                      ;Save w4 on to the stack

        mov     RXBUF0, w0              ;Perform dummy reads of the DCI
        mov     RXBUF1, w0              ;receive buffers -  RXBUF0 to RXBUF3
        mov     RXBUF2, w0
        mov     RXBUF3, w0

        mov     SineTablePtr, w4        ;Load w4 with contents of SineTablePtr
                                        ;i.e., point w4 to the next sample in
                                        ;the array named "SineTable"
        mov     #TXBUF0, w0             ;Load w0 with the address of TXBUF0
                                        ;i.e., point w0 to TXBUF0
              mov     #NUM_DCI_BUF, w3        ;w3 will be used as a counter
                                        ;Load w3 with number of DCI transmit
                                        ;buffers to be used for transmission
        mov     #SineTable+2*NUMSAMP, w2    ;Load w2 with END address of array
                                        ;named "SineTable" + 1
        mov     StepSize, w1            ;Load w1 with current value of StepSize
        cp0     w1                      ;Is w1 = 0?
        bra     nz, L0                  ;If not, then branch to L0
        mov     #SineTable, w4          ;else, initialize w4 to point to first
                                        ;sample in "SineTable"
L0:     mov     [w4], [w0++]            ;Load next sample from "SineTable" into
                                        ;next DCI transmit buffer, post-increment
                                        ;only the DCI transmit buffer pointer
        add     w4, w1, w4              ;Add the StepSize contained in w1 to
                                        ;w4, the pointer to "SineTable" array
        cpslt   w4, w2                  ;Has w4 reached the last sample in the
                                        ;"SineTable" array? (Is w4 > w2?) If so,
                                        ;then skip next instruction.
        mov     #SineTable, w4          ;Else, re-initialize w4 to the first
                                        ;sample in "SineTable"
        dec     w3, w3                  ;Decrement the DCI buffer counter, w3
        bra     nz, L0                  ;If w3 <> 0 then branch to L0
        mov     w4, SineTablePtr        ;Else, store w4 into "SineTablePtr"

exitDCIISR:                             ;Prepare to exit the DCI ISR
        pop     w4                      ;Restore w4 from stack
        pop.d   w2                      ;Restore w2 and w3 from stack
        pop.d   w0                      ;Restore w0 and w1 from stack
        bclr    IFS2, #DCIIF            ;Clear the DCI interrupt flag
        retfie





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        .end                                    ;EOF


