;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.60            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
; 1. This file contains the lab2_part3 subroutine that configures the dsPIC
;    UART2 module for polling-based transmission
; 2. This file also contains another subroutine named "diplay_string"
;
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab2.inc"             ;Lab-specific global equates

        .equ    BAUD, 57600             ;Equate for desired Baud Rate
        .equ    BAUD_RATE, (((FCY/BAUD)/16)-1) ;Formula for UART Baud Rate
                                        ;Register from Family Reference Manual

                                        ;Provide global definitions
        .global lab2_part3, display_string

        .section .data                  ;Allocate and initializr RAM space for
                                        ;initialized variables
String0:.byte 0x0A, 0x0D                ;0xA,0xD=Carriage Return and Line Feed
        .asciz "0 Hz Sine Transmitted !" ;".asciz" directive terminates an
                                        ;ascii string with a "null/zero"
String1:.byte 0x0A, 0x0D
        .asciz "225 Hz Sine Transmitted !"
String2:.byte 0x0A, 0x0D
        .asciz "450 Hz Sine Transmitted !"
String3:.byte 0x0A, 0x0D
        .asciz "900 Hz Sine Transmitted !"

        .section .text                  ;Code Section in Program memory
lab2_part3:
;UART2 Module Initialization
        clr     U2MODE                  ;Set SFRs to a known state
        clr     U2STA                   ;by clearing them

        ;U2BRG Initialization - Baud Rate Set up for transmission
        mov     #BAUD_RATE, w0          ;Initialize Baud Rate to 57.6 Kbaud
        mov     w0, U2BRG               ;calculated at the top of this file


        ;U2MODE Initialization - UART Mode setup
        bset    U2MODE, #UARTEN         ;Enable UART2 module

        ;U2STA Initialization - Transmitter set up
                                        ;Set up U2STA so that the UART -
        bset    U2STA, #UTXISEL         ;-Transmitter interrupts when a
        bset    U2STA, #UTXEN           ; character is transferred to the
                                        ; shift register and hence the
                                        ; transmit buffer becomes empty
                                        ;- Tranmitter is enabled


        ;UART2 Interrupts Set up
        bclr    IFS1, #U2RXIF           ;Clear UART2 Receiver interrupt flag
        bclr    IEC1, #U2RXIE           ;Disable UART2 Receiver interrupts
        bclr    IFS1, #U2TXIF           ;Clear UART2 transmit interrupt flag
        bclr    IEC1, #U2TXIE           ;Disable UART2 Transmit interrupts

        return                          ;Return to the calling routine
;...
;UART transmit buffer load Routine
;Name: display_string
;Scope: Global
;Notes:
;      This routine examines the "StepSize" variable to decide
;      which of 4 strings to display on the Hypererminal session
;      If StepSize=0, then w1 (the pointer to string) is loaded
;      with the address of the first character in "String0".
;      If StepSize=16, then w1 (the pointer to string) is loaded
;      with the address of the first character in "String1"
;      If StepSize=32, then w1 (the pointer to string) is loaded
;      with the address of the first character in "String2"
;      If StepSize=64, then w1 (the pointer to string) is loaded
;      with the address of the first character in "String3"
;
;      Once w1 contains the address of the string to be transmitted
;      the UART2 transmit FIFO buffer, U2TXREG, is loaded 4 chars at a time,
;      until the Null character is encountered.
;      Note that all 4 strings, String0-String3, are null-terminated.
;...
display_string:
        mov     StepSize, w0            ;Read StepSize into w0
        lsr     w0, #4, w0              ;Normalize w0 by shifting 4 bits
                                        ;to the right
        cp0     w0                      ;Is w0 = 0?
        bra     nz, L1                  ;No - go to L1
        mov     #String0, w1            ;Yes - then w1 = address of "String0"
        bra     L4
L1:     cp      w0, #1                  ;Is w0 = 1?
        bra     nz, L2                  ;No - go to L2
        mov     #String1, w1            ;Yes - then w1 = address of "String1"
        bra     L4
L2:     cp      w0, #2                  ;Is w0 = 2?
        bra     nz, L3                  ;No - go to L3
        mov     #String2, w1            ;Yes - then w1 = address of "String2"
        bra     L4
L3:     mov     #String3, w1            ;Else, w0=4, so w1=address of "String3"

L4:     btss    U2STA, #TRMT            ;Check if UART2 transmit buffers
        bra     L4                      ;are empty. If not then branch
                                        ;unconditionally to L4

        mov     #4, w2                  ;Load w2 with a count of 4
                                        ;w2 will be used to count how many UART
                                        ;transmit buffers have been filled.
L5:     mov.b   [w1++], w0              ;Load next character from the string
                                        ;into w0
        cp0.b   w0                      ;Check if w0 = 0, is it a "null" char?
        bra     z, L6                   ;If so then branch to L6
        mov.b   WREG, U2TXREG           ;else, load w0 into the UART2 transmit
                                        ;buffer
        dec     w2, w2                  ;Decrement the buffer usage counter, w2
        bra     nz, L5                  ;If not zero, then branch to L5 (next)
        bra     L4                      ;else, jump to L4 and wait for transmit
                                        ;operation to complete
L6:     return



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        .end                                    ;EOF




        