;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.60            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
; 1. This file contains the lab2_part1 subroutine that configures the dsPIC
;    12-bit A/D converter
; 2. This file also contains the A/D interrupt service routine (ISR) named -
;    "__ADCInterrupt"
;
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab2.inc"             ;Lab-specific global equates

                                        ;Provide global definitions
        .global lab2_part1              ;Provide global scope to "lab2_part2"
        .global __ADCInterrupt          ;Provide global scope to ADC ISR
                                        ;This is required so that the IVT
                                        ;is populated by the ADC ISR address.
        .global PotRP1, PotRP2, PotRP3  ;Provide global scope to data memory
        .global TempSensor, StepSize    ;variables that are accessed by other
                                        ;subroutines

        .section .data                  ;Allocate and initializr RAM space for
                                        ;initialized variables
PotRP1: .hword 0x0000                   ;Initialize "PotRP1" to 0x0000
PotRP2: .hword 0x0000                   ;Initialize "PotRP2" to 0x0000
PotRP3: .hword 0x0000                   ;Initialize "PotRP3" to 0x0000
TempSensor: .hword 0x0000               ;Initialize "TempSensor" to 0x0000
StepSize: .hword 0x0000                 ;Initialize "StepSize" to 0x0000


        .section .text                  ;Code Section in Program memory
lab2_part1:

;A/D Converter Module Initialization

        ;ADCON1 Initialization - Sampling and Conversion trigger selection
        mov     #0x00E4, w0             ;Set up ADCON1 so that the A/D module:
        mov     w0, ADCON1              ;Uses the Auto-sample and Auto-convert
                                        ;modes with default result formatting
                                        ;(unsigned integer)
                                        ;The A/D should be disabled for now

        ;ADCON2 Initialization  - Conversion sequencing and A/D buffer fill
        mov     #0x040C, w0             ;Set up ADCON2 so that the A/D module:
        mov     w0, ADCON2              ;-Uses AVdd and AVss as reference
                                        ;-Scans input selections dictated by
                                        ; the ADCSSL SFR
                                        ;-Interrupts the processor after 4
                                        ; conversions are completed
                                        ;-Uses one 16-word buffer
                                        ;-Always uses MUXA inputs

        ;ADCON3 Initialization  - Sampling Rate
        mov     #0x1F28, w0             ;Set up ADCON3 so that the A/D module:
        mov     w0, ADCON3              ;-Operates at a sampling rate of 8KHz
                                        ; using the system clock (not ADRC)
                                        ;To achieve this, proceed as follows:
                                        ;(a)First decide the sampling time.
                                        ;   For simplicity allocate maximum
                                        ;   sampling time using the SAMC bits.
                                        ;(b)Next, select the "Tad" period via
                                        ;   the ADCS bits so that:
                                        ;1/(Sampling Time + Conversion Time)=8K
                                        ;   Remember that,
                                        ;Conversion time = 14*Tad

        ;ADCHS Initialization   - Input selection for MuxA and MuxB
        mov     #0x0000, w0             ;ADCHS is a "don't care", when
        mov     w0, ADCHS               ;ALTS=0 and CSCNA=1 in ADCON2

        ;ADCSSL Initialization  - Input selection for scan sequence
        mov     #0x0170, w0             ;Select AN4, AN5, AN6 and AN8 as inputs
        mov     w0, ADCSSL              ;for the scan sequence

        ;ADPCFG Initialization - Pin configuration (Analog/Digital)
        mov     #0xFE8F, w0             ;Configure only AN4, AN5, AN6 and AN8
        mov     w0, ADPCFG              ;as analog pins - the rest should be
                                        ;digital

        ;A/D Interrupt Set up
        bclr    IFS0, #ADIF             ;Clear the A/D interrupt flag
        bset    IEC0, #ADIE             ;Enable A/D interrupts

        ;Turn On the A/D converter
        bset    ADCON1, #ADON           ;Enable the A/D module
		
        return                          ;Return to the calling subroutine

;...
;A/D Interrupt Service Routine
;Name: __ADCInterrupt
;Scope: Global
;FAQs:
;     1. Want to use a different name for your A/D ISR?
;        - Modify the name in the linker script
;     2. Why the double underscore?
;        - For compatibility with the C30 compiler & to avoid namespace issues
;     3. Why global scope for the ISR?
;        - So the linker can place the address into the IVT (Vector table)
;...
__ADCInterrupt:
        push.d  w0                      ;Save w0 and w1 on to stack

        mov     ADCBUF0, w0             ;Read ADCBUF0 and store to PotRP2
        mov     w0, PotRP2
        mov     ADCBUF1, w0             ;Read ADCBUF1 and store to PotRP3
        mov     w0, PotRP3
        mov     ADCBUF2, w0             ;Read ADCBUF2 and store to PotRP1
        mov     w0, PotRP1
        mov     ADCBUF3, w0             ;Read ADCBUF3 and store to TempSensor
        mov     w0, TempSensor

        ;Calculate Step Size for stepping through SineTable using PotRP1
        ;If 0 <= PotRP1 < 1024 then StepSize = 0
        ;If 1024 <= PotRP1 < 2048 then StepSize = 8 words
        ;If 2048 <= PotRP1 < 3072 then StepSize = 16 words
        ;If 3072 <= PotRP1 < 4095 then StepSize = 32 words

        mov     PotRP1, w0
        mov     #1023, w1
        cp      w0, w1
        bra     gt, L1
L0:     clr     w0
        bra     exitADCISR

L1:     mov     #2047, w1
        cp      w0, w1
        bra     gt, L2
        mov     #16, w0
        bra     exitADCISR

L2:     mov     #3071, w1
        cp      w0, w1
        bra     gt, L3
        mov     #32, w0
        bra     exitADCISR

L3:     mov     #64, w0

exitADCISR:
        mov     w0, StepSize            ;Store w0 into StepSize
        pop.d   w0                      ;Restore w0 and w1 from stack
        bclr    IFS0, #ADIF             ;Clear A/D interrupt flag
        retfie                          ;Return from Interupt



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        .end                                    ;EOF


