;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
; 1. This file contains a routine, lab3_init, that will configure the 
;    - Port D pins, RD0-RD7 as outputs, driving them high initially. RD0-RD3 
;      are connected to LEDs 1-4.
;    - Timer 2 to time out every 24.44 milliseconds (when using ECxPLL8 mode)
;      and generate an interrupt
; 2. This file also contains the Timer 2 Interrupt Service Routine that
;    toggles the state of the 4 LEDs on each Timer2 Interrupt.
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab3.inc"             ;Lab-specific global equates

        .global lab3_init               ;Provide global scope to routine
        .global __T2Interrupt           ;Provide global scope to the Timer2
                                        ;Interupt Service Routine(ISR)
                                        ;This ensures that the linker will
                                        ;place the address of the routine
                                        ;in the Interrupt Vector Table (IVT)

        .section .text
lab3_init:
        clr.b   TRISD                   ;Configure RD0-RD7 output pins
        setm.b  LATD                    ;Initialize these pins to a high
                                        ;Note that RD0-RD3 are connected to LEDs
                                        ;that are active Low

                                        ;Set up Timer 2 to time out every 24.44
                                        ;milliseconds
        clr     T2CON                   ;Clear the  Timer2 SFRs to a known state
        clr     TMR2
        bset    T2CON, #TCKPS0          ;Set the Timer 2 prescaler to 1:8
        mov     #0xAFFF, w0             ;Load the Timer2 Period Register so that
        mov     w0, PR2                 ;Value of Period Register*Prescaler/FCY
                                        ;provides 24.44 millisecond time-out

        bclr    IFS0, #T2IF             ;Clear the Timer2 Interrupt Flag
        bset    IEC0, #T2IE             ;Enable Timer2 ISR processing

        bset    T2CON, #TON             ;Turn on Timer2

        return                          ;Return to calling routine

;...
;Timer 2 Interrupt Service Routine
;...
__T2Interrupt:
        com.b   LATD                    ;Complement RD0-RD7 pins
        bclr    IFS0, #T2IF             ;Clear the Timer2 interrupt flag
        retfie                          ;Return from interrupt


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        .end                                            ;EOF



