;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
; 1. This file contains 4 interrupt service routines -one each for
;    INT1 - INT4 external interrupt pins
;    INT1 is used to request a switch to the Primary Oscillator
;    INT2 is used to request a switch to the FRC Oscillator
;    INT3 is used to request a switch to the LPRC Oscillator
;    INT4 is used to request a toggle the system clock postscaler
;    between /1 and /4 modes.
;
; 2. This file contains two routines - one each to perform an unlock and a byte
;    wide write operation on OSCCONH and OSCCONL respectively. These routines
;    are called by the INT1-INT4 ISRs.
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab3.inc"             ;Lab-specific global equates

        .global lab3_part1
        .global __INT1Interrupt, __INT2Interrupt
        .global __INT3Interrupt, __INT4Interrupt


        .section .text
lab3_part1:
;Configure External Interrupt Pins (INT1 - INT4)
        bclr IFS1, #INT1IF              ;Clear INT1-INT4 pin interrupt flag
        bclr IFS1, #INT2IF              ;status bits to bring them to a
        bclr IFS2, #INT3IF              ;known state (use IFSn SFRs)
        bclr IFS2, #INT4IF


        bset INTCON2, #INT1EP           ;Set up INT1-INT4 pins to detect
        bset INTCON2, #INT2EP           ;a falling edge on the pins
        bset INTCON2, #INT3EP           ;using INTCON2 SFR
        bset INTCON2, #INT4EP


        bset IEC1, #INT1IE              ;Enable Interrupt Service Routine
        bset IEC1, #INT2IE              ;processing for INT1 though INT4 pins
        bset IEC2, #INT3IE              ;(use IECn SFRs)
        bset IEC2, #INT4IE

        return

;...
;INT1 used to select Primary Osc as clock source
;...
__INT1Interrupt:
        push  w0                        ;Save w0 on to stack

        ; INSERT PART 1 CODE HERE       ;Place new OSCCONH value to into w0

		mov.b	#0x03, W0				; Added !!	
        rcall   WriteOSCCONH            ;Call subroutine to write to OSCCONH

        ; INSERT PART 1 CODE HERE       ;Place new OSCCONL value into w0

		mov.b	#0x01,W0					; Added !!
        rcall   WriteOSCCONL            ;Call subroutine to write to OSCCONL

        cp0     w0                      ;Check if the return value from
                                        ;WriteOSCCONL routine is = 0?
        mov     #MsgSwitchAbort, w0     ;Load w0 with the address of the
                                        ;Clock Switch Abort message
        bra     nz, $+4                 ;If Z bit is not set then
        mov     #MsgSwitchPri, w0       ;Load w0 with the address of the
                                        ;successful clock switch message
        rcall   _UpdateLCD              ;Call the _UpdateLCD routine

        pop     w0                      ;Restore w0
        bclr    IFS1, #INT1IF           ;Clear interrupt flag for INT1
        retfie                          ;Return from Interrupt


;...
;INT2 used to select FRC (8 MHz nominal) as clock source
;...
__INT2Interrupt:
        push  w0

        ; INSERT PART 1 CODE HERE       ;Place new OSCCONH value into w0
		mov.b	#0x01, W0					; Added !!
        rcall   WriteOSCCONH            ;Call subroutine to write to OSCCONH

        ; INSERT PART 1 CODE HERE       ;Place new OSCCONL value into w0
		mov.b	#0x01, W0
        rcall   WriteOSCCONL            ;Call subroutine to write to OSCCONL

        cp0     w0                      ;Check if the return value from
                                        ;WriteOSCCONL routine is = 0?
        mov     #MsgSwitchAbort, w0     ;Load w0 with the address of the
                                        ;Clock Switch Abort message
        bra     nz, $+4                 ;If Z bit is not set then
        mov     #MsgSwitchFRC, w0       ;Load w0 with the address of the
                                        ;successful clock switch message
        rcall   _UpdateLCD              ;Call the _UpdateLCD routine

        pop     w0                      ;Restore w0
        bclr    IFS1, #INT2IF           ;Clear interrupt flag for INT2
        retfie                          ;Return from Interrupt


;...
;INT3 used to select Internal LPRC (512 KHz nominal) as clock source
;...
__INT3Interrupt:
        push  w0

        ; INSERT PART 1 CODE HERE       ;Place new OSCCONH value into w0
		mov.b	#0x02 , W0				; Added !!
        rcall   WriteOSCCONH            ;Call subroutine to write to OSCCONH

        ; INSERT PART 1 CODE HERE       ;Place new OSCCONL value into w0
		mov.b	#0x01 , W0				; Added !!
        rcall   WriteOSCCONL            ;Call subroutine to write to OSCCONL

        cp0     w0                      ;Check if the return value from
                                        ;WriteOSCCONL routine is = 0?
        mov     #MsgSwitchAbort, w0     ;Load w0 with the address of the
                                        ;Clock Switch Abort message
        bra     nz, $+4                 ;If Z bit is not set then
        mov     #MsgSwitchLPRC, w0      ;Load w0 with the address of the
                                        ;successful clock switch message
        rcall   _UpdateLCD              ;Call the _UpdateLCD routine

        pop     w0                      ;Restore w0
        bclr    IFS2, #INT3IF           ;Clear interrupt flag for INT3
        retfie                          ;Return from Interrupt


;...
;INT4 used to toggle post-scale system clock by 4 or 1 (i.e. /4 or /1)
;Note - Other Oscillator Post Scaler options (/16, /64) not implemented in
;       this lab session
;...
__INT4Interrupt:
        push.d  w0                      ;Save w0, w1 on to stack

        clr     w0                      ;Clear w0 and load it with
        ; INSERT PART 1 CODE HERE       ;contents of the OSCCONL byte
        ; INSERT PART 1 CODE HERE       ;Toggle the POST0 bit position in w0
                                        ;and pass w0 as a parameter to the
		mov.b	OSCCONL, WREG			; Added !!
		btg		W0, #POST0				; Added !!

        rcall   WriteOSCCONL            ;WriteOSCCONL subroutine


        mov     #MsgSwitchDiv1, w0      ;Check POST0 bit in OSCCON to determine
        btsc    OSCCONL, #POST0         ;the new post-scaler value and load
        mov     #MsgSwitchDiv4, w0      ;w0 with the address of the appropriate
        rcall   _UpdateLCD              ;message, then call routine to
                                        ;update the LCD

        pop.d   w0                      ;Restore w0, w1 from stack
        bclr    IFS2, #INT4IF           ;Clear interrupt flag for INT4
        retfie                          ;Return from Interrupt

;...
;Subroutine to perform a write to OSCCON High Byte- OSCCONH
;Subroutine: WriteOSCCONH
;Parameters: Lower byte of w0 contains value to be written to OSCCONH
;Return Value: None
;...
WriteOSCCONH:
        push    w1                      ;Save w1, w2 and w3 on to stack
        push.d  w2

        mov     #OSCCONH, w1            ;Point w1 to the address of OSCCONH
        ; INSERT PART 1 CODE HERE       ;Disable interrupts when performing
                                        ;OSCCONL (low byte) unlock sequence,
                                        ;i.e., for next 5 instruction cycles
		disi	#4

        ; INSERT PART 1 CODE HERE       ;Move first OSCCONH unlock code byte
                                        ;to w2
		mov		#0x78, W2
        ; INSERT PART 1 CODE HERE       ;Move second OSCCONH unlock code byte
                                        ;to w3
		mov		#0x9A, W3
        ; INSERT PART 1 CODE HERE       ;Perform byte-wide move of w2 to OSCCONH
        ; INSERT PART 1 CODE HERE       ;Perform byte-wide move of w3 to OSCCONH
		
		mov.b	W2,[W1]
		mov.b	W3,[W1]

        ; INSERT PART 1 CODE HERE       ;Write parameter in w0 (lower byte)to
                                        ;OSCCONH within one instruction cycle
                                        ;of unlocking OSCCONH
		mov.b	W0,[W1]

        pop.d   w2                      ;Restore w1, w2 and w3 from stack
        pop     w1
        return

;...
;Subroutine to perform a write to OSCCON Low Byte- OSCCONL
;Subroutine: WriteOSCCONL
;Parameters: Lower byte of w0 contains value to be written to OSCCONL
;Return Value: w0=0x0000 for successful clock switch
;              w0=0xFFFF for unsuccessful clock switch
;...
WriteOSCCONL:
        push    w1                      ;Save w1, w2 & w3 on to stack
        push.d  w2
        push    w4

        mov     #OSCCONL, w1            ;Point w1 to the address of OSCCONL
        ; INSERT PART 1 CODE HERE       ;Disable interrupts when performing
                                        ;OSCCONL (low byte) unlock sequence,
                                        ;i.e., for next 5 instruction cycles
		disi	#4

        ; INSERT PART 1 CODE HERE       ;Move first OSCCONL unlock code byte
                                        ;to w2
		mov		#0x46, W2
        ; INSERT PART 1 CODE HERE       ;Move second OSCCONL unlock code byte
                                        ;to w3
		mov		#0x57, w3
        ; INSERT PART 1 CODE HERE       ;Perform byte-wide move of w2 to OSCCONL
        ; INSERT PART 1 CODE HERE       ;Perform byte-wide move of w3 to OSCCONL
		
		mov.b	w2,[w1]
		mov.b	w3,[w1]

        mov.b   w0, [w1]                ;Write parameter in w0 (lower byte)to
                                        ;OSCCONL within one instruction cycle
                                        ;of unlocking OSCCONL
                                        ;If clock switch is being performed and
                                        ;OSWEN has been set, wait for a finite
                                        ;time for the switch to complete (OSWEN
                                        ;to get cleared). If OSWEN does not get
                                        ;cleared then abort the clock switch
        mov     #0xFFFF, w0             ;Load a value into w0 (example 0xFFFF)
L0:     dec     w0, w0                  ;Decrement w0
        bra     z, AbortClkSwitch       ;If w0 = 0, then abort the clock switch
                                        ;procedure (jump to AbortClkSwitch),
        ; INSERT PART 1 CODE HERE       ;else, check the status of the OSWEN bit

		btsc	OSCCON, #OSWEN
        bra     L0                      ;and skip next instruction if OSWEN has
                                        ;been cleared.
                                        ;If clock switch is not being performed
                                        ;in this write to OSCCONL byte then
                                        ;OSWEN will be 0 already, so exit
                                        ;routine!
        clr     w0                      ;Load return value into w0 indicating
                                        ;a successful clock switch (=0)
L1:     pop     w4
        pop.d   w2                      ;Restore w1, w2 & w3 from stack
        pop     w1
        return

AbortClkSwitch:                         ;Code execution reaches here if
                                        ;we need to abort an ongoing Clock-
                                        ;switching procedure
        ; INSERT PART 1 CODE HERE       ;Load w0 with a value to write into
		mov.b	#0x00, W0
                                        ;OSCCONL (OSWEN bit must be cleared)
        mov     #OSCCONL, w1            ;Point w1 to the address of OSCCONL
        disi    #4                      ;Disable interrupts when performing
                                        ;OSCCONL (low byte) unlock sequence,
                                        ;i.e., for next 5 instruction cycles
		

        mov     #0x46, w2               ;Move first OSCCONL unlock code byte
                                        ;to w2
        mov     #0x57, w3               ;Move second OSCCONL unlock code byte
                                        ;to w3
        mov.b   w2, [w1]                ;Perform byte-wide move of w2 to OSCCONL
        mov.b   w3, [w1]                ;Perform byte-wide move of w2 to OSCCONL

        mov.b   w0, [w1]                ;Write parameter in w0 (lower byte)to
                                        ;OSCCONL within one instruction cycle
                                        ;of unlocking OSCCONL
        setm    w0                      ;Load return value into w0 indicating
                                        ;an unsuccessful clock switch (=0xFFFF)
        bra     L1                      ;Exit subroutine


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        .end                                            ;EOF



