;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
; 1. This file contains the Oscillator Fail Trap Handler routine
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab3.inc"             ;Lab-specific global equates

        .global __OscillatorFail        ;Provide global scope to the Oscillator
                                        ;Failure Trap Handler; this ensures that
                                        ;the linker will place the address of
                                        ;the routine in the Interrupt Vector
                                        ;Table


        .section .text                  ;Code section in Program Memory
__OscillatorFail:
        ; INSERT PART 2 CODE HERE       ;Save w0, w1, w2 and w3 on to stack
                                        ;using the single level of shadow
                                        ;registers and a Special Instruction

                                        ;On encountering a clock failure, if
                                        ;FSCM is enabled, device executes using
                                        ;the FRC oscillator and code execution
                                        ;reaches the Oscillatore Fail trap
                                        ;Before clearing the trap status flag
                                        ;we should clear the Clock Fail bit in
                                        ;OSCCONL
        ; INSERT PART 2 CODE HERE       ;Load w0 with a byte value to write to
                                        ;OSCCONL so that the CF bit is cleared
        mov     #OSCCONL, w1            ;Point w1 to OSCCONL byte
        disi    #4                      ;Disable interrupts for the next
                                        ;5 instruction cycles
        mov     #0x46, w2               ;Move first OSCCONL unlock code byte
                                        ;to w2
        mov     #0x57, w3               ;Move second OSCCONL unlock code byte
                                        ;to w3
        mov.b   w2, [w1]                ;Perform byte-wide move of w2 to OSCCONL
        mov.b   w3, [w1]                ;Perform byte-wide move of w2 to OSCCONL

        mov.b   w0, [w1]                ;Write parameter in w0 (lower byte)to
                                        ;OSCCONL within one instruction cycle
                                        ;of unlocking OSCCONL

        mov.b   #0x0A, W0               ;Write a new pattern into the LEDs
        mov.b   WREG, LATD              ;connected to Port D to indicate
                                        ;Oscillator Failure

        ; INSERT PART 2 CODE HERE       ;Clear the Oscillator Failure Trap
                                        ;flag bit

        clr.b   SRL                     ;In order to update LCD via SPI
                                        ;first lower the priority level
                                        ;of the CPU by clearing the IPL bits
                                        ;When a trap is encountered the CPU
                                        ;is placed on a priority level > 7
        mov     #MsgClockFail, w0       ;Load w0 with the address of the
        rcall   _UpdateLCD              ;Clock Fail Massage string and call the
                                        ;routine to update the LCD


        pop.s                           ;Restore w0, w1, w2 and w3 from stack
                                        ;using a Special Instruction
        retfie                          ;Return from interrupt


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        .end                                            ;EOF



