;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
;
; File Notes:
; 1. SPI2 ISR routine has two modes of operation:
;    a. In the normal mode, it transmits one control byte 0xA8 and one
;       data (ASCII) byte alternately until the char count has reached zero.
;       0xA8 increments ChrPos to the next location.
;    b. In some cases, ChrPos increments are not required. based on the value
;       of a flag variable, the control byte transmission may be skipped.
;
;END_HEADER


	.include "p30f6014.inc"



	.global __SPI2Interrupt		;Provide global scope
	.global LCDDataPtr, CtrlOrData, LcdCharCnt, BypassCtrl

	.section 	.bss
LcdCharCnt: 		.space 2

	.section 	.data
CtrlOrData: 		.hword 0xFFFF
LCDDataPtr: 		.hword Hello	;.hword LCDString
BypassCtrl:		.hword 0x0000
	
	.section .text			;Places source code in Code section
	
__SPI2Interrupt:

        push.d w0               	;Context save

        bset LATG, #LATG9       	;Deselect Slave LCD controller
        bclr IFS1, #SPI2IF      	;clear interrupt flag
        bclr SPI2STAT, #SPIROV  	;Clear overflow flag
        mov.b SPI2BUF, WREG     	;perform read of received byte  buffer count


        cp0 LcdCharCnt          	;Check if the character count has reached 0
        bra z, exit_SPI2_isr	   	;If so, then exit ISR

        cp0 BypassCtrl          	;Check if "A8" Ctrl chars need to be bypassed
        bra nz, bypassCtrlChars 	;If so, don't check whether next byte is control
                                	;or data byte

        cp0 CtrlOrData          	;Else, check if byte to be transmitted is
                                	;control or data byte
        bra nz, FetchCtrlByte   	;If it is, then fetch control byte 0xA8

        setm CtrlOrData         	;Else, set flag to control byte for next trx

bypassCtrlChars:
        mov LCDDataPtr, w1      	;mov data byte address to w1
        mov.b [w1++], w0        	;read the byte
        dec LcdCharCnt          	;decrement count
        mov w1, LCDDataPtr      	;Update tx byte address for next transmission
        bclr LATG, #LATG9       	; SS_: Enable SPI reception by LCD controller
        mov.b WREG, SPI2BUF     	;Send byte to LCD controller

exit_SPI2_isr:
        pop.d w0               		;restore context
        retfie                  	;return from interrupt

FetchCtrlByte:
        clr CtrlOrData          	;Clear flag for data trx

        mov.b #0xA8, w0         	;mov control byte to w0
        bclr LATG, #LATG9       	; SS_: Enable SPI reception by LCD controller
        mov.b WREG, SPI2BUF     	;Send byte to LCD controller
	bra exit_SPI2_isr



	


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.end					;EOF

