;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
;
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab4.inc"             ;Lab-specific global equates

        .equ    PERIOD_1MS, FCY/1000
        .equ    DUTYCYCLE_50PC_1MS, PERIOD_1MS/2

        .global lab4_init
        .global __AltT2Interrupt, __T2Interrupt

        .section .text                  ;Places source code in Code section
lab4_init:
        rcall   _port_pins_init         ;Configure port pins multiplexed with
                                        ;Input Capture and Output Compare
                                        ;modules

        rcall   _SPI2_init              ;"Relative" Call to SPI2
                                        ;initialaization routine

        rcall   _LCD_Controller_init    ;"Relative" Call to PIC18F242
                                        ;initialization routine

        rcall   _T1_init                ;Configure Timer 1 for use as an
                                        ;interval timer to update LCD display

        rcall   _ADC_init               ;Configure A/D to convert AN6
                                        ;AN6 is connected to Potentiometer RP1
                                        ;on the expansion board. RP1 is used to
                                        ;vary the PWM frequency and pulse width

        rcall   _OC8PWM_init            ;Configure OC module for PWM operation


        rcall   _T2_init                ;Configure Timer2 as a timebase for
                                        ;Output Compare PWM

        return


_T2_init:
        push    w0                      ;Save context

        clr     T2CON                   ;Set module to known state
        clr     TMR2                    ;Ensure Timer2 counter is reset

        mov     #PERIOD_1MS, w0         ;Initialize Period Register 2
        mov     w0, PR2                 ;for PWM Period to 1 millisecond
                                        ;on power-up

        bclr    IFS0, #T2IF             ;Clear Timer2 interrupt flag
        bset    IEC0, #T2IE             ;Enable Timer2 ISR processing
        bset    T2CON, #TON             ;Turn on PWM timebase - Timer2

        pop     w0                      ;Restore context
        return

__AltT2Interrupt:
__T2Interrupt:
        push.d  w0                      ;Save context of w0, w1

        ;Reset TMR3 SFR on a Timer2 Interrupt so that Timer3 does not roll-over
        ;This ensures that all Timer3 captured values reflect the time period
        ;of a the PWM signal.
        mov     #7, w0                  ;Reload the Input Capture Timebase,
        mov     w0, TMR3                ;TMR3, with constant 0x0007

        ;Update the PWM frequency with new RP1 value available on ADCBUF0
        ;while maintaining the duty cycle of the PWM pulse at 50%

        mov     ADCBUF0, w0             ;Read the Potentiometer RP1 conversion
                                        ;from ADCBUF0
        mov     #10, w1                 ;Check if RP1 conversion < 10
        cp      w0, w1
        bra     ge, updatePR            ;Load the new OC/PWM timebase period
                                        ;into w0
        mov     w1, w0                  ;If RP1 < 10, load w0 with 10
                                        ;This ensures that we do not load the
                                        ;timebase period register with 0x0000
updatePR:
        sl      w0, #4, w0              ;Shift left the 12-bit ADC conversion
                                        ;in w0 by 4 bits
        mov     w0, PR2                 ;Update PR2 with contents of w0
        lsr     w0, w0                  ;Load PR2/2 into OC8RS for 50%
        mov     w0, OC8RS               ;duty cycle

        pop.d w0                        ;Restore context of w0, w1
        bclr    IFS0, #T2IF             ;Clear the Timer2 Inetrrupt Flag bit
        retfie                          ;Return from Interrupt

_OC8PWM_init:
        push    w0

        clr     OC8CON                  ;Bring the OC8CON SFR to known state
        bclr    OC8CON, #OCTSEL         ;Set Timer 2 as the timebase

        mov     #DUTYCYCLE_50PC_1MS, w0 ;Set initial value for 50% duty cycle
        mov     w0, OC8R                ;OC8R is write-able when module is off
        mov     w0, OC8RS               ;Write duty cycle update val. into OC8RS

        bclr    IFS2, #OC8IF            ;Clear interrupt flag bit
        bclr    IEC2, #OC8IE            ;Disable ISR processing

        bset    OC8CON, #OCM2           ;Set the compare mode to produce
        bset    OC8CON, #OCM1           ;glitchless PWM output at the
        bclr    OC8CON, #OCM0           ;OC8/RD7 pin

        pop     w0
        return

_port_pins_init:
; Set up pin RD7 as an Output pin to enable Output Compare-PWM operation on OC8
        bclr    LATD, #LATD7            ;Clear the LATD bit - LATD7
        bclr    TRISD, #TRISD7          ;Clear the TRISD bit - TRISD7

; Set up pin RD13 as an Input pin to enable Input capture operation on IC6
        bclr    LATD, #LATD13           ;Clear the LATD bit - LATD7
        bset    TRISD, #TRISD7          ;Set the TRISD bit - TRISD13

        return                          ;Return to calling routine




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.end                                            ;EOF

