;START_HEADER
;
; dsPIC30F Training Source File, Revision 1.0
; (c) Copyright 2004 Microchip Technology, All rights reserved
;
; File Revision History:
; -----------------------------------------------------------------------------
; Rev.  MM/DD/YYYY   Details:                                           Who
; -----------------------------------------------------------------------------
; 1.0   06/30/2004   Lab Session for Masters 806 P30
;
;
; -----------------------------------------------------------------------------
;
; Software and Development Tools Info:
; -----------------------------------------------------------------------------
; Tool                          Version         Comments
; -----------------------------------------------------------------------------
; MPLAB IDE                     6.51            Required for development/ICD2
; MPLAB C30 Toolsuite           1.20.02         Required for assembler, linker
; dsPICDEM Board                1.1             Required hardware
; dsPIC30F6014 PIM              Rev B1/B2       Required processor
; -----------------------------------------------------------------------------
;
; File Notes:
;
;..............................................................................
;END_HEADER
                                        ;Include definition files
        .include "p30fxxxx.inc"         ;Device specific SFR bit definitions
        .include "lab4.inc"             ;Lab-specific global equates

        .equ    MAX_PERIOD, 0xFFFF

        .global __AltIC6Interrupt       ;Provide global scope to routine
        .global lab4_part2              ;Provide global scope to routine

        .section .data
EdgeCount:      .hword 0x0000           ;EdgeCount keeps a count of edges

        .section .text                  ;Places source code in Code section
lab4_part2:
        clr   T3CON                     ;Ensure Timer3 module is in a known
                                        ;state
        clr   TMR3                      ;Ensure timer3 counter is reset


        mov   #MAX_PERIOD, w0           ;Load the maximum 16-bit count into
        mov   w0, PR3                   ;PR3

        bclr  IFS0, #T3IF               ;Reset Timer3 interrupt flag

        bclr  IEC0, #T3IE               ;Disable Timer3 ISR processing
                                        ;Timer3 is free-running

        bset    T3CON, #TON             ;Turn on Capture time-base - Timer3

        clr     IC6CON                  ;Clear the IC6CON SFR to a known
                                        ;state

        bclr    IC6CON, #ICM2           ;Configure IC6 to capture Timer3
        bclr    IC6CON, #ICM1           ;value on every edge using Bit
        bset    IC6CON, #ICM0           ;instructions
                                        ;NOTE: ICI bits have no effect

        bclr    IFS1, #IC6IF            ;Clear the IC6 interrupt flag bit
        bset    IEC1, #IC6IE            ;Enable IC6 ISR processing

        setm    FreqOrPulseDisplay      ;When set, this flag informs LCD display
                                        ;routine to display pulse width
                                        ;information

        bset    INTCON2, #ALTIVT        ;Enable Alternate ISR processing
                                        ;Alternate ISR is used only for Input
                                        ;Capture module. Main and Alternate ISR
                                        ;labels for remaining peripherals are
                                        ;associated in source code with the same
                                        ;physical ISR
        return



__AltIC6Interrupt:
        push.d  w0                      ;Save context of w0-w3
        push.d  w2

        inc     EdgeCount               ;Increment EdgeCount variable

        ;If EdgeCount < 2 then exit ISR
        mov     #0x0002, w0             ;Load w0 with 2
        cp      EdgeCount               ;Compare Edgecount against w0
        bra     nz, exit                ;If EdgeCount <> w0 then exit ISR

        mov     IC6BUF, w0              ;Else move the two captured
        mov     IC6BUF, w1              ;values into w0 and w1

        sub     w1, w0, w0              ;Subtract w0 from w1 and save into w0
        mov     w0, PulseWidth          ;Move w0 to variable "PulseWidth"

        clr     EdgeCount               ;Clear EdgeCount to 0x0000
        setm    DisplayFlag             ;Set the LCD DsplayFlag

exit:
        pop.d   w2                      ;Restore context of w0 - w3
        pop.d   w0
        bclr    IFS1, #IC6IF            ;Clear the IC6 interrupt flag bit
        retfie

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                    ;
;                       Software License Agreement                   ;
;                                                                    ;
;   The software supplied herewith by Microchip Technology           ;
;   Incorporated (the "Company") for its dsPIC controller            ;
;   is intended and supplied to you, the Company's customer,         ;
;   for use solely and exclusively on Microchip dsPIC                ;
;   products. The software is owned by the Company and/or its        ;
;   supplier, and is protected under applicable copyright laws. All  ;
;   rights are reserved. Any use in violation of the foregoing       ;
;   restrictions may subject the user to criminal sanctions under    ;
;   applicable laws, as well as to civil liability for the breach of ;
;   the terms and conditions of this license.                        ;
;                                                                    ;
;   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           ;
;   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    ;
;   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    ;
;   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     ;
;   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  ;
;   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  ;
;                                                                    ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.end                                            ;EOF



